"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.useBootstrapPrefix = useBootstrapPrefix;
exports.useClassNameMapper = useClassNameMapper;
exports.createBootstrapComponent = createBootstrapComponent;
exports.default = exports.ThemeConsumer = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _forwardRef = _interopRequireDefault(require("@restart/context/forwardRef"));

var _react = _interopRequireWildcard(require("react"));

var _createClassNames = _interopRequireDefault(require("./createClassNames"));

var ThemeContext = _react.default.createContext({
  prefixes: {},
  classNameMap: {},
  createClassNameMapper: function createClassNameMapper(classMap) {
    return (0, _createClassNames.default)(classMap, {}, function (item) {
      return item;
    });
  }
});

var Consumer = ThemeContext.Consumer,
    Provider = ThemeContext.Provider;
exports.ThemeConsumer = Consumer;

function ThemeProvider(_ref) {
  var prefixes = _ref.prefixes,
      classNameMap = _ref.classNameMap,
      classNameConverter = _ref.classNameConverter,
      children = _ref.children;
  var copiedPrefixes = (0, _react.useMemo)(function () {
    return (0, _extends2.default)({}, prefixes);
  }, [prefixes]);
  var globalClassNameMap = (0, _react.useMemo)(function () {
    return (0, _extends2.default)({}, classNameMap);
  }, [classNameMap]);

  var createClassNameMapper = function createClassNameMapper(localClassNameMap) {
    return (0, _createClassNames.default)(localClassNameMap, globalClassNameMap, classNameConverter);
  };

  return _react.default.createElement(Provider, {
    value: {
      prefixes: copiedPrefixes,
      classNameMap: globalClassNameMap,
      createClassNameMapper: createClassNameMapper
    }
  }, children);
}

function useBootstrapPrefix(prefix, defaultPrefix) {
  var _useContext = (0, _react.useContext)(ThemeContext),
      prefixes = _useContext.prefixes;

  return prefix || prefixes[defaultPrefix] || defaultPrefix;
}

function useClassNameMapper(localClassNameMap) {
  var _useContext2 = (0, _react.useContext)(ThemeContext),
      createClassNameMapper = _useContext2.createClassNameMapper;

  return createClassNameMapper(localClassNameMap);
}

function createBootstrapComponent(Component, opts) {
  if (typeof opts === 'string') opts = {
    prefix: opts
  };
  var isClassy = Component.prototype && Component.prototype.isReactComponent; // If it's a functional component make sure we don't break it with a ref

  var _opts = opts,
      prefix = _opts.prefix,
      _opts$forwardRefAs = _opts.forwardRefAs,
      forwardRefAs = _opts$forwardRefAs === void 0 ? isClassy ? 'ref' : 'innerRef' : _opts$forwardRefAs;
  return (0, _forwardRef.default)(function (_ref2, ref) {
    var props = (0, _extends2.default)({}, _ref2);
    props[forwardRefAs] = ref; // eslint-disable-next-line react/prop-types

    var bsPrefix = useBootstrapPrefix(props.bsPrefix, prefix);

    var _useContext3 = (0, _react.useContext)(ThemeContext),
        classNameMap = _useContext3.classNameMap;

    return _react.default.createElement(Component, (0, _extends2.default)({}, props, {
      bsPrefix: bsPrefix,
      classNameMap: classNameMap
    }));
  }, {
    displayName: "Bootstrap(" + (Component.displayName || Component.name) + ")"
  });
}

var _default = ThemeProvider;
exports.default = _default;