import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";

/* eslint-disable react/no-multi-comp */
import React from 'react';
import SafeAnchor from './SafeAnchor';
import { useClassNameMapper } from './ThemeProvider';
var defaultProps = {
  active: false,
  disabled: false,
  activeLabel: '(current)'
};
var PageItem = React.forwardRef(function (_ref, ref) {
  var active = _ref.active,
      disabled = _ref.disabled,
      className = _ref.className,
      classNameMap = _ref.classNameMap,
      style = _ref.style,
      activeLabel = _ref.activeLabel,
      children = _ref.children,
      props = _objectWithoutPropertiesLoose(_ref, ["active", "disabled", "className", "classNameMap", "style", "activeLabel", "children"]);

  var Component = active || disabled ? 'span' : SafeAnchor;
  var classNames = useClassNameMapper(classNameMap);
  return React.createElement("li", {
    ref: ref,
    style: style,
    className: classNames(className, 'page-item', {
      active: active,
      disabled: disabled
    })
  }, React.createElement(Component, _extends({
    className: classNames('page-link'),
    disabled: disabled
  }, props), children, active && activeLabel && React.createElement("span", {
    className: classNames('sr-only')
  }, activeLabel)));
});
PageItem.defaultProps = defaultProps;
PageItem.displayName = 'PageItem';
export default PageItem;

function createButton(name, defaultValue, label) {
  if (label === void 0) {
    label = name;
  }

  var Button = function Button(_ref2) {
    var classNameMap = _ref2.classNameMap,
        children = _ref2.children,
        props = _objectWithoutPropertiesLoose(_ref2, ["classNameMap", "children"]);

    return React.createElement(PageItem, props, React.createElement("span", {
      "aria-hidden": "true"
    }, children || defaultValue), React.createElement("span", {
      className: useClassNameMapper(classNameMap)('sr-only')
    }, label));
  };

  Button.displayName = name;
  return Button;
}

export var First = createButton('First', '«');
export var Prev = createButton('Prev', '‹', 'Previous');
export var Ellipsis = createButton('Ellipsis', '…', 'More');
export var Next = createButton('Next', '›');
export var Last = createButton('Last', '»');